#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"

#define SYSCALL_NOT_IMPL		0x3C60800160630003ULL
#define SYSCALL_NOT_IMPL2		0x4E800020
#define LV2_BEGIN_SEEK			0x8000000000000000ULL
#define LV2_END_SEEK			0x8000000000600000ULL



uint64_t lv2_peek(uint64_t addr)
{
    system_call_1(6, addr);
	return_to_user_prog(uint64_t);
}


void lv2_poke(uint64_t addr, uint64_t val)
{
    system_call_2(7, addr, val);
}

void lv1_poke(uint64_t addr, uint64_t val)
{
    system_call_2(9, addr, val);
}

uint64_t find_syscall()
{
	uint64_t i = LV2_END_SEEK;

	while(i>LV2_BEGIN_SEEK)
	{
		if(lv2_peek(i) == SYSCALL_NOT_IMPL)
			if(((lv2_peek(i+8) >> 32) & 0xFFFFFFFF) == SYSCALL_NOT_IMPL2)
				return i;
		i-=4;
	}
	return 0;
}

uint64_t reverse_search64(uint64_t val)
{
	uint64_t i = LV2_END_SEEK;

	while(i>LV2_BEGIN_SEEK)
	{
		if(lv2_peek(i) == val)
			return i;
		i-=4;
	}
	return 0;
}

uint64_t search64(uint64_t val)
{
	uint64_t i;

	for(i=LV2_BEGIN_SEEK;i<LV2_END_SEEK;i+=4)
	{
		if(lv2_peek(i) == val)
			return i;
	}
	return 0;
}

uint64_t find_syscall_table()
{
	uint64_t sc, opd_sc;

	sc = find_syscall();
	opd_sc = reverse_search64(sc);
	return search64(opd_sc);
}

int get_lv2_version()
{
	uint64_t toc = lv2_peek(0x8000000000003000ULL);

	switch(toc)
	{
		case 0x8000000000346390ULL:
			return 0x421C;
		case 0x8000000000363E80ULL:
			return 0x421D;
		case 0x8000000000366BD0ULL:
			return 0x446D;
		case 0x8000000000348DF0ULL:
			return 0x446C;
		case 0x800000000034B160ULL:
			return 0x450C;
		case 0x800000000036EC40ULL:
			return 0x450D;
		case 0x800000000034B2E0ULL:
			return 0x453C;
		case 0x800000000034E620ULL:
			return 0x455C;
		case 0x800000000034F950ULL:
			return 0x460C;
		default:
			return 0;
	}
}

uint64_t get_syscall_table()
{
	int version = get_lv2_version();
	switch(version)
	{
		case 0x421C:
			return 0x800000000035BCA8ULL;
		case 0x421D:
			return 0x800000000037A1B0ULL;
		case 0x446C:
			return 0x800000000035E860ULL;
		case 0x446D:
			return 0x800000000037CFE8ULL;
		case 0x450C:
			return 0x800000000035F0D0ULL;
		case 0x450D:
			return 0x8000000000383658ULL;
		case 0x453C:
			return 0x800000000035F300ULL;
		case 0x455C:
			return 0x8000000000362680ULL;
		case 0x460C:
			return 0x8000000000363A18ULL;
	}

	return 0;
}


int install_syscall(int syscall_number, uint64_t *payload, uint32_t payload_size, uint64_t install_offset)
{
	uint64_t syscall_table = get_syscall_table();
	uint64_t payload_opd = install_offset + payload_size + 0x10;
	int i;
	
	if(syscall_table)
	{
		for(i=0;i<(payload_size/8);i++)
			lv2_poke(install_offset+(i*8), payload[i]);

		lv2_poke(payload_opd, install_offset);
		lv2_poke(syscall_table + (8*syscall_number), payload_opd);
		//lv2_poke(syscall_table + (8*1021), 0x8000000000324DF0ULL); //memcpy 4.46dex
		return 1;
	}

	return 0;
}

uint64_t find_vsh_process_obj()
{
	uint64_t vsh_str = reverse_search64(0x5F6D61696E5F7673ULL);
	return lv2_peek((vsh_str + 0x70));
}


int lv2_memcpy(uint64_t target, uint64_t source, uint32_t size)
{
	system_call_3(1021, target, source, size);
	return_to_user_prog(uint64_t);
}

void dump_lv2(char name[])
{
	//uint8_t buffer[0x1000];
	uint64_t i, val;

	FILE * out = fopen(name, "wb");

	for(i=0x8000000000000000;i<0x8000000000800000;i+=8)
	{
		val = lv2_peek(i);
		fwrite(&val, 8, 1, out);
	}

	fclose(out);

}

void write_htab(void)
{
    uint64_t cont = 0;
    uint64_t reg5, reg6;
    uint32_t val;

    while(cont < 0x80) 
	{
        val = (cont << 7);

        reg5 = lv2_peek(0x800000000f000000ULL | ((uint64_t) val));
        reg6 = lv2_peek(0x800000000f000008ULL | ((uint64_t) val));
        reg6 = (reg6  & 0xff0000ULL) | 0x190ULL;

		system_call_8(10, 0, (cont << 3ULL), reg5, reg6, 0, 0, 0, 1);

        cont++;

    }
}

void lv2_patch_error(uint64_t lis_addr, uint32_t lis_instr, uint64_t ori_addr, uint32_t ori_instr)
{
	uint64_t val;
	val = (lv2_peek(lis_addr) & 0xFFFFFFFF) | ((uint64_t)lis_instr << 32);
	lv2_poke(lis_addr, val);

	val = (lv2_peek(ori_addr) & 0xFFFFFFFF) | ((uint64_t)ori_instr << 32);
	lv2_poke(ori_addr, val);
}

